==> 2205.0.tar.gz: o3de/Code/Tools/AssetProcessor/native/tests/utilities/JobModelTest.cpp <==
/*
 * Copyright (c) Contributors to the Open 3D Engine Project.
 * For complete copyright and license terms please see the LICENSE at the root of this distribution.
 *
 * SPDX-License-Identifier: Apache-2.0 OR MIT
 *
 */

#include <native/tests/utilities/JobModelTest.h>

TEST_F(JobModelUnitTests, Test_RemoveMiddleJob)
{
    VerifyModel(); // verify up front for sanity.

    AzToolsFramework::AssetSystem::JobInfo jobInfo;
    jobInfo.m_sourceFile = "source2.txt";
    jobInfo.m_platform = "platform";
    jobInfo.m_jobKey = "jobKey";
    AssetProcessor::QueueElementID elementId("source2.txt", "platform", "jobKey");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 1); // second job
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 6);
    m_unitTestJobModel->OnJobRemoved(jobInfo);
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 5);
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_EQ(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    AssetProcessor::CachedJobInfo* cachedJobInfo = m_unitTestJobModel->m_cachedJobs[jobIndex];
    ASSERT_EQ(cachedJobInfo->m_elementId.GetInputAssetName(), QString("source3.txt"));
    // Checking index of last job
    elementId.SetInputAssetName("source6.txt");
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 4);
    VerifyModel();
}

TEST_F(JobModelUnitTests, Test_RemoveFirstJob)
{
    VerifyModel(); // verify up front for sanity.

    AzToolsFramework::AssetSystem::JobInfo jobInfo;
    jobInfo.m_sourceFile = "source1.txt";
    jobInfo.m_platform = "platform";
    jobInfo.m_jobKey = "jobKey";
    AssetProcessor::QueueElementID elementId("source1.txt", "platform", "jobKey");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 0); //first job
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 6);
    m_unitTestJobModel->OnJobRemoved(jobInfo);
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 5);
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_EQ(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    AssetProcessor::CachedJobInfo* cachedJobInfo = m_unitTestJobModel->m_cachedJobs[jobIndex];
    ASSERT_EQ(cachedJobInfo->m_elementId.GetInputAssetName(), QString("source2.txt"));
    // Checking index of last job
    elementId.SetInputAssetName("source6.txt");
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 4);
    VerifyModel();
}

TEST_F(JobModelUnitTests, Test_RemoveLastJob)
{
    VerifyModel(); // verify up front for sanity.

    AzToolsFramework::AssetSystem::JobInfo jobInfo;
    jobInfo.m_sourceFile = "source6.txt";
    jobInfo.m_platform = "platform";
    jobInfo.m_jobKey = "jobKey";
    AssetProcessor::QueueElementID elementId("source6.txt", "platform", "jobKey");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 5); //last job
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 6);
    m_unitTestJobModel->OnJobRemoved(jobInfo);
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 5);
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_EQ(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    AssetProcessor::CachedJobInfo* cachedJobInfo = m_unitTestJobModel->m_cachedJobs[jobIndex - 1];
    ASSERT_EQ(cachedJobInfo->m_elementId.GetInputAssetName(), QString("source5.txt"));
    // Checking index of first job
    elementId.SetInputAssetName("source1.txt");
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 0);
    VerifyModel();
}


TEST_F(JobModelUnitTests, Test_RemoveAllJobsBySource)
{
    VerifyModel(); // verify up front for sanity.

    AssetProcessor::CachedJobInfo* jobInfo1 = new AssetProcessor::CachedJobInfo();
    jobInfo1->m_elementId.SetInputAssetName("source3.txt"); // this is the second job for this source - the fixture creates one
    jobInfo1->m_elementId.SetPlatform("platform_2"); // differing job keys
    jobInfo1->m_elementId.SetJobDescriptor("jobKey_3"); // differing descriptor

    jobInfo1->m_jobState = AzToolsFramework::AssetSystem::JobStatus::Completed;
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo1);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo1->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::QueueElementID elementId("source3.txt", "platform_2", "jobKey_3");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 6); //last job

    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 7);
    m_unitTestJobModel->OnSourceRemoved("source3.txt");

    // both  sources should be removed!
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 5);
    VerifyModel();

    // make sure source3 is completely gone.
    for (int idx = 0; idx < m_unitTestJobModel->m_cachedJobs.size(); idx++)
    {
        AssetProcessor::CachedJobInfo* jobInfo = m_unitTestJobModel->m_cachedJobs[idx];
        ASSERT_NE(jobInfo->m_elementId.GetInputAssetName(), QString::fromUtf8("source3.txt"));
    }
}

TEST_F(JobModelUnitTests, Test_RemoveAllJobsBySourceFolder)
{
    VerifyModel(); // verify up front for sanity.

    AssetProcessor::CachedJobInfo* testJobInfo = new AssetProcessor::CachedJobInfo();
    testJobInfo->m_elementId.SetInputAssetName("sourceFolder1/source.txt");
    testJobInfo->m_elementId.SetPlatform("platform");
    testJobInfo->m_elementId.SetJobDescriptor("jobKey");

    testJobInfo->m_jobState = AzToolsFramework::AssetSystem::JobStatus::Completed;
    m_unitTestJobModel->m_cachedJobs.push_back(testJobInfo);
    m_unitTestJobModel->m_cachedJobsLookup.insert(testJobInfo->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::QueueElementID elementId("sourceFolder1/source.txt", "platform", "jobKey");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 6); //last job

    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 7);
    m_unitTestJobModel->OnFolderRemoved("sourceFolder1");

    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 6);
    VerifyModel();

    // make sure sourceFolder1/source.txt is completely gone.
    for (int idx = 0; idx < m_unitTestJobModel->m_cachedJobs.size(); idx++)
    {
        AssetProcessor::CachedJobInfo* jobInfo = m_unitTestJobModel->m_cachedJobs[idx];
        ASSERT_NE(jobInfo->m_elementId.GetInputAssetName(), QString::fromUtf8("sourceFolder1/source.txt"));
    }
}

void JobModelUnitTests::SetUp()
{
    AssetProcessorTest::SetUp();

    m_unitTestJobModel = new UnitTestJobModel();
    AssetProcessor::CachedJobInfo* jobInfo1 = new AssetProcessor::CachedJobInfo();
    jobInfo1->m_elementId.SetInputAssetName("source1.txt");
    jobInfo1->m_elementId.SetPlatform("platform");
    jobInfo1->m_elementId.SetJobDescriptor("jobKey");
    jobInfo1->m_jobState = AzToolsFramework::AssetSystem::JobStatus::Completed;
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo1);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo1->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo2 = new AssetProcessor::CachedJobInfo();
    jobInfo2->m_elementId.SetInputAssetName("source2.txt");
    jobInfo2->m_elementId.SetPlatform("platform");
    jobInfo2->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo2);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo2->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo3 = new AssetProcessor::CachedJobInfo();
    jobInfo3->m_elementId.SetInputAssetName("source3.txt");
    jobInfo3->m_elementId.SetPlatform("platform");
    jobInfo3->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo3);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo3->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo4 = new AssetProcessor::CachedJobInfo();
    jobInfo4->m_elementId.SetInputAssetName("source4.txt");
    jobInfo4->m_elementId.SetPlatform("platform");
    jobInfo4->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo4);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo4->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo5 = new AssetProcessor::CachedJobInfo();
    jobInfo5->m_elementId.SetInputAssetName("source5.txt");
    jobInfo5->m_elementId.SetPlatform("platform");
    jobInfo5->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo5);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo5->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo6 = new AssetProcessor::CachedJobInfo();
    jobInfo6->m_elementId.SetInputAssetName("source6.txt");
    jobInfo6->m_elementId.SetPlatform("platform");
    jobInfo6->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo6);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo6->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));
}

void JobModelUnitTests::TearDown()
{
    delete m_unitTestJobModel;

    AssetProcessorTest::TearDown();
}

void JobModelUnitTests::VerifyModel()
{
    // Every job should exist in the lookup map as well.
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), m_unitTestJobModel->m_cachedJobsLookup.size());

    // every job in the vector should have a corresponding element in the lookup table.
    for (int idx = 0; idx < m_unitTestJobModel->m_cachedJobs.size(); idx++)
    {
        AssetProcessor::CachedJobInfo* jobInfo = m_unitTestJobModel->m_cachedJobs[idx];
        auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(jobInfo->m_elementId);
        ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    }

    // this tests the other direction - every job in the lookup table should map to a job in the vector
    // we also verify that its the appropriate job and not an off-by-one type of problem.
    for (const auto& key : m_unitTestJobModel->m_cachedJobsLookup.keys())
    {
        int expectedIndex = m_unitTestJobModel->m_cachedJobsLookup[key];
        ASSERT_LT(expectedIndex, m_unitTestJobModel->m_cachedJobs.size());
        ASSERT_EQ(m_unitTestJobModel->m_cachedJobs[expectedIndex]->m_elementId, key);
    }
}

==> 2210.0.tar.gz: o3de/Code/Tools/AssetProcessor/native/tests/utilities/JobModelTest.cpp <==
/*
 * Copyright (c) Contributors to the Open 3D Engine Project.
 * For complete copyright and license terms please see the LICENSE at the root of this distribution.
 *
 * SPDX-License-Identifier: Apache-2.0 OR MIT
 *
 */

#include <native/tests/utilities/JobModelTest.h>
#include <AzCore/std/string/string.h>

TEST_F(JobModelUnitTests, Test_RemoveMiddleJob)
{
    VerifyModel(); // verify up front for sanity.

    AzToolsFramework::AssetSystem::JobInfo jobInfo;
    jobInfo.m_sourceFile = "source2.txt";
    jobInfo.m_platform = "platform";
    jobInfo.m_jobKey = "jobKey";
    AssetProcessor::QueueElementID elementId("source2.txt", "platform", "jobKey");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 1); // second job
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 6);
    m_unitTestJobModel->OnJobRemoved(jobInfo);
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 5);
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_EQ(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    AssetProcessor::CachedJobInfo* cachedJobInfo = m_unitTestJobModel->m_cachedJobs[jobIndex];
    ASSERT_EQ(cachedJobInfo->m_elementId.GetInputAssetName(), QString("source3.txt"));
    // Checking index of last job
    elementId.SetInputAssetName("source6.txt");
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 4);
    VerifyModel();
}

TEST_F(JobModelUnitTests, Test_RemoveFirstJob)
{
    VerifyModel(); // verify up front for sanity.

    AzToolsFramework::AssetSystem::JobInfo jobInfo;
    jobInfo.m_sourceFile = "source1.txt";
    jobInfo.m_platform = "platform";
    jobInfo.m_jobKey = "jobKey";
    AssetProcessor::QueueElementID elementId("source1.txt", "platform", "jobKey");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 0); //first job
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 6);
    m_unitTestJobModel->OnJobRemoved(jobInfo);
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 5);
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_EQ(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    AssetProcessor::CachedJobInfo* cachedJobInfo = m_unitTestJobModel->m_cachedJobs[jobIndex];
    ASSERT_EQ(cachedJobInfo->m_elementId.GetInputAssetName(), QString("source2.txt"));
    // Checking index of last job
    elementId.SetInputAssetName("source6.txt");
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 4);
    VerifyModel();
}

TEST_F(JobModelUnitTests, Test_RemoveLastJob)
{
    VerifyModel(); // verify up front for sanity.

    AzToolsFramework::AssetSystem::JobInfo jobInfo;
    jobInfo.m_sourceFile = "source6.txt";
    jobInfo.m_platform = "platform";
    jobInfo.m_jobKey = "jobKey";
    AssetProcessor::QueueElementID elementId("source6.txt", "platform", "jobKey");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 5); //last job
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 6);
    m_unitTestJobModel->OnJobRemoved(jobInfo);
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 5);
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_EQ(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    AssetProcessor::CachedJobInfo* cachedJobInfo = m_unitTestJobModel->m_cachedJobs[jobIndex - 1];
    ASSERT_EQ(cachedJobInfo->m_elementId.GetInputAssetName(), QString("source5.txt"));
    // Checking index of first job
    elementId.SetInputAssetName("source1.txt");
    iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 0);
    VerifyModel();
}


TEST_F(JobModelUnitTests, Test_RemoveAllJobsBySource)
{
    VerifyModel(); // verify up front for sanity.

    AssetProcessor::CachedJobInfo* jobInfo1 = new AssetProcessor::CachedJobInfo();
    jobInfo1->m_elementId.SetInputAssetName("source3.txt"); // this is the second job for this source - the fixture creates one
    jobInfo1->m_elementId.SetPlatform("platform_2"); // differing job keys
    jobInfo1->m_elementId.SetJobDescriptor("jobKey_3"); // differing descriptor

    jobInfo1->m_jobState = AzToolsFramework::AssetSystem::JobStatus::Completed;
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo1);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo1->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::QueueElementID elementId("source3.txt", "platform_2", "jobKey_3");
    auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
    ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    unsigned int jobIndex = iter.value();
    ASSERT_EQ(jobIndex, 6); //last job

    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 7);
    m_unitTestJobModel->OnSourceRemoved("source3.txt");

    // both  sources should be removed!
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), 5);
    VerifyModel();

    // make sure source3 is completely gone.
    for (int idx = 0; idx < m_unitTestJobModel->m_cachedJobs.size(); idx++)
    {
        AssetProcessor::CachedJobInfo* jobInfo = m_unitTestJobModel->m_cachedJobs[idx];
        ASSERT_NE(jobInfo->m_elementId.GetInputAssetName(), QString::fromUtf8("source3.txt"));
    }
}

TEST_F(JobModelUnitTests, Test_PopulateJobsFromDatabase)
{
    VerifyModel(); // verify up front for sanity.

    CreateDatabaseTestData();
    m_unitTestJobModel->PopulateJobsFromDatabase();

    for (const auto& jobEntry : m_data->m_jobEntries)
    {
        AssetProcessor::QueueElementID elementId(m_data->m_sourceName.c_str(), jobEntry.m_platform.c_str(), jobEntry.m_jobKey.c_str());
        auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(elementId);
        ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());

        //! expect the three jobs from database are populated into m_unitTestJobModel
        EXPECT_EQ(m_unitTestJobModel->m_cachedJobs.at(iter.value())->m_elementId, elementId);

        //! and that they have a valid m_processDuration, which is set to be equivalent to jobEntry.m_fingerprint
        EXPECT_EQ(
            m_unitTestJobModel->m_cachedJobs.at(iter.value())->m_processDuration.msecsSinceStartOfDay(),
            aznumeric_cast<int>(jobEntry.m_fingerprint));
    }

    // expect one AZ_Warning emitted, because we have one stat entry in the database that has 5 tokens.
    m_errorAbsorber->ExpectWarnings(1);

    VerifyModel();
}

void JobModelUnitTests::SetUp()
{
    AssetProcessorTest::SetUp();

    {
        using namespace testing;
        m_data.reset(new StaticData());

        //! Setup temporary database
        m_data->m_temporaryDatabaseDir = QDir(m_data->m_temporaryDir.path());
        QString canonicalTempDirPath = AssetUtilities::NormalizeDirectoryPath(m_data->m_temporaryDatabaseDir.canonicalPath());
        m_data->m_temporaryDatabaseDir = QDir(canonicalTempDirPath);
        m_data->m_temporaryDatabasePath = m_data->m_temporaryDatabaseDir.absoluteFilePath("test_database.sqlite").toUtf8().data();

        //! Setup Asset Database connection.
        //! There will be two database connection: one for CreateDatabaseTestData, and one inside UnitTestJobModel::PopulateJobsFromDatabase.
        //! In-memory databases can have only a single connection, so we need a file-backed SQLite database to serve the needs.
        m_data->m_databaseLocationListener.BusConnect();
        ON_CALL(m_data->m_databaseLocationListener, GetAssetDatabaseLocation(_))
            .WillByDefault(DoAll(
                SetArgReferee<0>(m_data->m_temporaryDatabasePath.c_str()),
                Return(true)));
        m_data->m_connection.ClearData();
    }

    m_unitTestJobModel = new UnitTestJobModel();
    AssetProcessor::CachedJobInfo* jobInfo1 = new AssetProcessor::CachedJobInfo();
    jobInfo1->m_elementId.SetInputAssetName("source1.txt");
    jobInfo1->m_elementId.SetPlatform("platform");
    jobInfo1->m_elementId.SetJobDescriptor("jobKey");
    jobInfo1->m_jobState = AzToolsFramework::AssetSystem::JobStatus::Completed;
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo1);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo1->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo2 = new AssetProcessor::CachedJobInfo();
    jobInfo2->m_elementId.SetInputAssetName("source2.txt");
    jobInfo2->m_elementId.SetPlatform("platform");
    jobInfo2->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo2);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo2->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo3 = new AssetProcessor::CachedJobInfo();
    jobInfo3->m_elementId.SetInputAssetName("source3.txt");
    jobInfo3->m_elementId.SetPlatform("platform");
    jobInfo3->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo3);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo3->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo4 = new AssetProcessor::CachedJobInfo();
    jobInfo4->m_elementId.SetInputAssetName("source4.txt");
    jobInfo4->m_elementId.SetPlatform("platform");
    jobInfo4->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo4);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo4->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo5 = new AssetProcessor::CachedJobInfo();
    jobInfo5->m_elementId.SetInputAssetName("source5.txt");
    jobInfo5->m_elementId.SetPlatform("platform");
    jobInfo5->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo5);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo5->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));

    AssetProcessor::CachedJobInfo* jobInfo6 = new AssetProcessor::CachedJobInfo();
    jobInfo6->m_elementId.SetInputAssetName("source6.txt");
    jobInfo6->m_elementId.SetPlatform("platform");
    jobInfo6->m_elementId.SetJobDescriptor("jobKey");
    m_unitTestJobModel->m_cachedJobs.push_back(jobInfo6);
    m_unitTestJobModel->m_cachedJobsLookup.insert(jobInfo6->m_elementId, aznumeric_caster(m_unitTestJobModel->m_cachedJobs.size() - 1));
}

void JobModelUnitTests::TearDown()
{
    delete m_unitTestJobModel;

    m_data->m_databaseLocationListener.BusDisconnect();
    m_data.reset();

    AssetProcessorTest::TearDown();
}

void JobModelUnitTests::VerifyModel()
{
    // Every job should exist in the lookup map as well.
    ASSERT_EQ(m_unitTestJobModel->m_cachedJobs.size(), m_unitTestJobModel->m_cachedJobsLookup.size());

    // every job in the vector should have a corresponding element in the lookup table.
    for (int idx = 0; idx < m_unitTestJobModel->m_cachedJobs.size(); idx++)
    {
        AssetProcessor::CachedJobInfo* jobInfo = m_unitTestJobModel->m_cachedJobs[idx];
        auto iter = m_unitTestJobModel->m_cachedJobsLookup.find(jobInfo->m_elementId);
        ASSERT_NE(iter, m_unitTestJobModel->m_cachedJobsLookup.end());
    }

    // this tests the other direction - every job in the lookup table should map to a job in the vector
    // we also verify that its the appropriate job and not an off-by-one type of problem.
    for (const auto& key : m_unitTestJobModel->m_cachedJobsLookup.keys())
    {
        int expectedIndex = m_unitTestJobModel->m_cachedJobsLookup[key];
        ASSERT_LT(expectedIndex, m_unitTestJobModel->m_cachedJobs.size());
        ASSERT_EQ(m_unitTestJobModel->m_cachedJobs[expectedIndex]->m_elementId, key);
    }
}

void JobModelUnitTests::CreateDatabaseTestData()
{
    //! This method puts jobs and ProcessJob metrics into the database.

    using namespace AzToolsFramework::AssetDatabase;
    ScanFolderDatabaseEntry scanFolderEntry;
    SourceDatabaseEntry sourceEntry;
    StatDatabaseEntry statEntry;

    scanFolderEntry = { "c:/O3DE/dev", "dev", "rootportkey" };
    ASSERT_TRUE(m_data->m_connection.SetScanFolder(scanFolderEntry));
    sourceEntry = { scanFolderEntry.m_scanFolderID, m_data->m_sourceName.c_str(), AZ::Uuid::CreateRandom(), "AFPAFPAFP1" };
    ASSERT_TRUE(m_data->m_connection.SetSource(sourceEntry));

    //! Insert job entries
    m_data->m_jobEntries.clear();
    m_data->m_jobEntries.emplace_back(
        sourceEntry.m_sourceID, "jobKey1", 123, "pc", AZ::Uuid::CreateRandom(), AzToolsFramework::AssetSystem::JobStatus::Completed, 1);
    m_data->m_jobEntries.emplace_back(
        sourceEntry.m_sourceID, "jobKey2", 456, "linux", AZ::Uuid::CreateRandom(), AzToolsFramework::AssetSystem::JobStatus::Failed, 2);
    m_data->m_jobEntries.emplace_back(
        sourceEntry.m_sourceID, "jobKey3", 789, "mac", AZ::Uuid::CreateRandom(), AzToolsFramework::AssetSystem::JobStatus::Completed, 3);
    for (auto& jobEntry : m_data->m_jobEntries)
    {
        ASSERT_TRUE(m_data->m_connection.SetJob(jobEntry));
    }

    //! Insert valid stat entries, one per job
    for (const auto& jobEntry : m_data->m_jobEntries)
    {
        AZStd::string statName = AZStd::string::format(
            "ProcessJob,%s,%s,%s,%s",
            m_data->m_sourceName.c_str(),
            jobEntry.m_jobKey.c_str(),
            jobEntry.m_platform.c_str(),
            jobEntry.m_builderGuid.ToString<AZStd::string>().c_str());
        statEntry = { statName /* StatName */,
                      aznumeric_cast<AZ::s64>(jobEntry.m_fingerprint) /* StatValue */,
                      aznumeric_cast<AZ::s64>(jobEntry.m_jobRunKey) /* LastLogTime */ };
        ASSERT_TRUE(m_data->m_connection.ReplaceStat(statEntry));
    }

    //! Insert an invalid stat entry (6 tokens)
    statEntry = { "ProcessJob,apple,banana,carrot,dog,{FDAF4363-C530-476C-B382-579A43B3E2FC}", 123, 456 };
    ASSERT_TRUE(m_data->m_connection.ReplaceStat(statEntry));
}
